package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.QmsQueueResultMapper;
import com.steerinfo.dil.mapper.QmsQueueSpellingListMapper;
import com.steerinfo.dil.mapper.QueuingRulesMapper;
import com.steerinfo.dil.model.QmsQueueResult;
import com.steerinfo.dil.model.QmsQueueSpellingList;
import com.steerinfo.dil.model.WebSocketCapacity;
import com.steerinfo.dil.util.DataChange;
import com.steerinfo.dil.model.QmsQueueList;
import com.steerinfo.dil.mapper.QmsQueueListMapper;
import com.steerinfo.dil.service.IQmsQueueListService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;
import java.math.BigDecimal;

/**
 * QmsQueueList服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-14 11:29
 * 类描述
 * 修订历史：
 * 日期：2021-09-14
 * 作者：generator
 * 参考：
 * 描述：QmsQueueList服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "qmsQueueListService")
public class QmsQueueListServiceImpl implements IQmsQueueListService {

    @Autowired
    private QmsQueueListMapper qmsQueueListMapper;

    @Autowired
    private QmsQueueResultMapper qmsQueueResultMapper;

    @Autowired
    private QueuingRulesServiceImpl queuingRulesService;

    @Autowired
    private QueuingRulesMapper  queuingRulesMapper;

    @Autowired
    private QmsQueueSpellingListMapper qmsQueueSpellingListMapper;


    private final SimpleDateFormat gatepostTimeSdf = new SimpleDateFormat("HHmmss");

    /**
     * 根据时间计算钢材进厂门岗（钢材只有东门、桥西门能进厂）
     * @return
     */
    public BigDecimal calculateGatepost(){
        //查询东门，桥西门的进厂时间规则
        try{
            List<Map<String, Object>> gatepostRules = qmsQueueListMapper.getGatepostRules();
            String nowTime = gatepostTimeSdf.format(new Date());
            for (Map<String, Object> gatepostRule : gatepostRules) {
                if(gatepostRule.get("startTime") != null && gatepostRule.get("endTime") != null){
                    boolean judge = judgeTime(String.valueOf(gatepostRule.get("startTime")),
                                    String.valueOf(gatepostRule.get("endTime")), nowTime);
                    if(judge)
                        return  DataChange.dataToBigDecimal(gatepostRule.get("gatepostId"));
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return new BigDecimal(2);//默认桥西门
    }

    /**
     * 判断当前时间是否在时间区间范围内
     * @param startTime
     * @param endTime
     * @return
     */
    public boolean judgeTime(String startTime, String endTime, String nowTime){
        int begin = Integer.parseInt(startTime);
        int end = Integer.parseInt(endTime);
        int now = Integer.parseInt(nowTime);
        if(begin < end){
            return now < end && now >= begin;
        }else { //跨天
            return now < end || now >= begin;
        }
    }

    /**
     * 开始排队  添加 链表排序 司机点击确认
     * @param resultId 排队实绩ID
     * @return
     */
    @Transactional
    public int addQueueList(BigDecimal resultId){
        //通过订单查询有几种物资
        Map<String, Object> mesMap = qmsQueueListMapper.getMaterialNumByOrderId(resultId);
        //放入排队的门岗
        int materialNum = DataChange.dataToBigDecimal(mesMap.get("materialNum")).intValue();
        if(materialNum == 1){
            //单拼进入链表
            //判断单拼链表中是否有已存在此辆车正在排队
            if(qmsQueueListMapper.existsCapacity(mesMap.get("capacityId")) != null){
                return 0;
            }
            return addListResult(mesMap, calculateGatepost());
        }else {
            //多拼进入排队队列
            if(qmsQueueSpellingListMapper.existsCapacity(mesMap.get("capacityId")) != null){
                return 0;
            }
            return addSpellingListResult(mesMap, calculateGatepost());
        }
    }

    /**
     * 添加链表
     * @param map
     * @return
     */
    public int addListResult(Map<String, Object> map, BigDecimal gatepostId){
        QmsQueueList qmsQueueList = new QmsQueueList();
        //添加主键ID
        BigDecimal maxId = qmsQueueListMapper.selectMaxId();
        qmsQueueList.setListId(maxId);
        BigDecimal resultId = DataChange.dataToBigDecimal(map.get("resultId"));
        qmsQueueList.setQueueResultId(resultId);
        //添加网格ID、运力ID
        qmsQueueList.setGridId(DataChange.dataToBigDecimal(map.get("gridId")));
        qmsQueueList.setCapacityId(DataChange.dataToBigDecimal(map.get("capacityId")));
        //查询当前链表还在排队中的 最大序号 + 1
        Integer maxListNodeOrder = qmsQueueListMapper.getMaxListNodeOrder(map);
        int nodeOrder = (maxListNodeOrder == null ? 0 : maxListNodeOrder) + 1;
        //添加排队序号与逻辑删除 0： 正在排队
        qmsQueueList.setListNodeOrder(new BigDecimal(nodeOrder));
        //添加常规字段
        qmsQueueList.setInsertUsername("admin");
        qmsQueueList.setInsertTime(new Date());
        qmsQueueListMapper.insertSelective(qmsQueueList);
        //更新排队实绩表
        updateQQR(resultId, maxId, gatepostId, DataChange.dataToBigDecimal(map.get("resultTotalId")));
        return 1;
    }

    /**
     * 添加多拼排序
     * @param map
     * @return
     */
    public int addSpellingListResult(Map<String, Object> map, BigDecimal gatepostId){
        QmsQueueSpellingList qmsQueueSpellingList = new QmsQueueSpellingList();
        BigDecimal maxId = qmsQueueSpellingListMapper.selectMaxId();
        qmsQueueSpellingList.setId(maxId);
        //排队实绩ID
        BigDecimal resultId = DataChange.dataToBigDecimal(map.get("resultId"));
        qmsQueueSpellingList.setQueueResultId(resultId);
        qmsQueueSpellingList.setSpelilingNum(DataChange.dataToBigDecimal(map.get("materialNum")));
        qmsQueueSpellingList.setSpellingSureTime(new Date()); //确认时间
        qmsQueueSpellingList.setInsertTime(new Date());
        qmsQueueSpellingList.setInsertUsername("admin");
        //新增排序实绩
        qmsQueueSpellingListMapper.insertSelective(qmsQueueSpellingList);
        //更新排队实绩表
        updateQQR(resultId, maxId, gatepostId,DataChange.dataToBigDecimal(map.get("resultTotalId")));
        return 1;
    }

    /**
     *  更新排队实绩表
     * @param resultId 实绩ID
     * @param listId 链表Id
     */
    public void updateQQR(BigDecimal resultId, BigDecimal listId, BigDecimal gatepostId, BigDecimal resultTotalId) {
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        qmsQueueResult.setListId(listId);
        qmsQueueResult.setResultId(resultId);
        if(gatepostId != null){
            qmsQueueResult.setGatepostId(gatepostId); // 门岗ID
            //更新路段门岗
            qmsQueueListMapper.udpateEnFactoryGatepostId(resultTotalId, gatepostId);
        }
        qmsQueueResult.setResultAcceptTime(new Date()); //添加接收时间
        qmsQueueResult.setResultStartTime(new Date()); //添加排队开始时间
        qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
    }


    /**
     * 取消排队
     *  传入参数：
     *  运输订单号 或 排队实绩ID
     *  取消原因
     *  如果是司机：通过运输订单ID
     *  如果是调度人员：可通过实绩Id
     * @param map
     * @return
     */
    @Override
    @Transactional
    public int queueCancel(Map<String, Object> map) {
        int i = 0;
        if(map.get("resultIdList") != null){
            Object resultCancelReason = map.get("resultCancelReason");
            List<Map<String, Object>> resultIdList = (List) map.get("resultIdList");
            for (Map<String, Object> objectMap : resultIdList) {
                objectMap.put("resultCancelReason", resultCancelReason);
                i += queueCancelSon(objectMap);
            }
        }else if(map.get("phone") != null){
            i = queueCancelSon(map);
        }
        return i;
    }

    /**
     * 取消子表
     * @param map
     * @return
     */
    public int queueCancelSon(Map<String, Object> map){
        BigDecimal resultId = DataChange.dataToBigDecimal(map.get("resultId"));
        //移出队列
        qmsQueueListMapper.deleteByQueueResultId(resultId);
        qmsQueueSpellingListMapper.deleteByQueueResultId(resultId);
        //更新排队实绩
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        qmsQueueResult.setResultId(resultId);
        qmsQueueResult.setResultCancelReason((String) map.get("resultCancelReason"));//设置取消原因
        return qmsQueueResultMapper.updateCancelResult(qmsQueueResult);
    }

    /**
     * 排队插队 通过实绩Id进行关联
     * 实绩ID
     * 插队原因
     * @param map
     * @return
     */
    @Override
    public int queueCutInLine(Map<String, Object> map) {
        //通过实绩Id查出关联的链表Id
        map.putAll(qmsQueueListMapper.getQueueResultByOrderNumber(map));
        //获取链表Id
        BigDecimal listId = DataChange.dataToBigDecimal(map.get("listId"));
        QmsQueueList qmsQueueList = new QmsQueueList();
        //设置链表排队顺序号 为 最小的那个 -1
        //获取排队顺序号最小值
        BigDecimal gridId = qmsQueueListMapper.getGridIdByListId(listId);
        //通过网格Id匹配当前列表中最小的顺序号
        map.putAll(qmsQueueListMapper.getQueueNumAndMin(gridId));
        //计算插队后顺序号
        int listNodeOrder = DataChange.dataToBigDecimal(map.get("listNodeOrder")).intValue() - 1;
        qmsQueueList.setListId(listId);
        qmsQueueList.setListNodeOrder(new BigDecimal(listNodeOrder));
        qmsQueueList.setUpdateTime(new Date());
        qmsQueueList.setUpdateUsername("admin");
        int i = qmsQueueListMapper.updateByPrimaryKeySelective(qmsQueueList);
        //更新排队实绩
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        qmsQueueResult.setResultId(DataChange.dataToBigDecimal(map.get("resultId")));
        qmsQueueResult.setResultInsertTime(new Date());
        qmsQueueResult.setResultInsertReason((String)map.get("resultInsertReason"));
        int i1 = qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
        return i + i1;
    }

    /**
     * 进厂排队结束 PDA 形式
     * 运输订单号
     * @param resultTotalId
     * @return
     */
    @Override
    @Transactional
    public int queueEndByPDA(BigDecimal resultTotalId) {
        //通过总实绩ID查询排队实绩ID以及是否拼装
        Map<String, Object> mesMap = qmsQueueResultMapper.getResultMesByTotalId(resultTotalId);
        BigDecimal resultId = DataChange.dataToBigDecimal(mesMap.get("resultId"));
        int isSpelling = DataChange.dataToBigDecimal(mesMap.get("isSpelling")).intValue();
        //将此数据移出队列
        if(isSpelling == 0){
            qmsQueueListMapper.deleteByQueueResultId(resultId);
        }else if(isSpelling == 1){
            qmsQueueSpellingListMapper.deleteByQueueResultId(resultId);
        }
        //给排队实绩中添加数据
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        qmsQueueResult.setResultId(resultId);
        qmsQueueResult.setResultEndTime(new Date());
        qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
        return 1;
    }

    /**
     * 允许车辆进厂（需优化对数据库访问,时间不够）
     * @param map
     * @return
     */
    public int allowEnFactory(Map<String, Object> map){
        List<Map<String, Object>> mapList = (List<Map<String, Object>>) map.get("mapList"); //获取勾选列表
        String ssoId = String.valueOf(map.get("ssoId"));
        String jobName = null;
        if(!"null".equals(ssoId)){
            //通过SSOID在人员表里查询相关人员账号 （唯一）
            jobName = queuingRulesMapper.getPersonnelNameBySSoId(ssoId);
        }
        int count = 0;
        ArrayList<Object> list = new ArrayList<>();
        ArrayList<BigDecimal> existsTotalIdList = new ArrayList<>();
        for (Map<String, Object> mesMap : mapList) {
            if(mesMap.get("sureTime") != null){
                continue;   //如果已经确认过了就不继续处理
            }
            mesMap.put("jobName", jobName);
            if(mesMap.get("listId") != null){ //单拼
                //更新排队链表
                count += qmsQueueListMapper.updateListEnFactory(mesMap);
                //更新进厂实绩
                qmsQueueListMapper.updateEnFactoryResult(mesMap);
                list.add(new WebSocketCapacity(5 , "system",String.valueOf(mesMap.get("capacityNumber")), "可以进厂" + new Date(), new Date()));
            }else{
                BigDecimal resultTotalId = DataChange.dataToBigDecimal(mesMap.get("resultTotalId"));
                if( ! existsTotalIdList.contains(resultTotalId)) {
                    //更新排序表
                    count += qmsQueueSpellingListMapper.updateListEnFactory(mesMap);
                    existsTotalIdList.add(resultTotalId);
                    //更新进厂实绩
                    qmsQueueListMapper.updateEnFactoryResult(mesMap);
                    list.add(new WebSocketCapacity(5 , "system",String.valueOf(mesMap.get("capacityNumber")), "可以进厂" + new Date(), new Date()));
                }
            }
            //更新物资子表 每个物资的装货点ID
            qmsQueueSpellingListMapper.updateOOMLoadWarehouse(mesMap);
        }
        queuingRulesService.pushMesToWebsocket(list);
        return count;
    }


    /**
     * 链表监控
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> listMonitor(Map<String, Object> map) {
        return qmsQueueListMapper.listMonitor(map);
    }

    public List<Map<String, Object>> spellingListMonitor(Map<String, Object> map) {
        return qmsQueueListMapper.spellingListMonitor(map);
    }

    /**
     * 链表查看详情
     * @param map
     * @return
     */
    public List<Map<String, Object>> getListMonitorMes(Map<String, Object> map){
        return qmsQueueResultMapper.getQueueListByQueueUp(map);
    }

    public List<Map<String, Object>> getSpellingListMonitorMes(Map<String, Object> map){
        return qmsQueueResultMapper.getQueueListByQueueUpSpelling(map);
    }

    /**
     * 排队转移
     * 参数：实绩Id、门岗名称||门岗ID 或者 月台名称||月台ID、转移原因
     * @param map
     * @return
     */
//    @Override
//    @Transactional
//    public int changeQueue(Map<String, Object> map) {
//        return 1;
//    }

    /**
     * 结束仓库排队
     * @param map {orderNumber 运输订单号}
     * @return
     */
//    public int queueEndWarehouse(Map<String, Object> map){
//        //通过运输订单号查询排队实绩ID 链表Id
//        map.putAll(qmsQueueListMapper.getQueueResultByOrderNumber(map));
//        BigDecimal resultId = DataChange.dataToBigDecimal(map.get("resultId")); //实绩Id
//        BigDecimal listId = DataChange.dataToBigDecimal(map.get("listId"));  //链表Id
//        //通过链表id修改链表对应数据 将此数据移出队列 deleted = 1
//        int i = updateQueueListToDie(listId);
//        //给排队实绩中添加数据
//        QmsQueueResult qmsQueueResult = new QmsQueueResult();
//        qmsQueueResult.setResultId(resultId);
//        qmsQueueResult.setResultEndTime(new Date());
//        int i1 = qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
//        return i + i1;
//    }

    /**
     * 仓库排队监控
     * @param map
     * @return
     */
//    @Override
//    public List<Map<String, Object>> warehouseListMonitor(Map<String, Object> map) {
//        return qmsQueueListMapper.warehouseListMonitor(map);
//    }

    /**
     * 查询排队详情 门岗ID 或者 月台ID
     * @param map
     * @return
     */
//    @Override
//    public List<Map<String, Object>> getListQueueMes(Map<String, Object> map) {
//        Map<String, Object> hashMap = new HashMap<>();
//        if(map.get("gatepostId") != null){//传入的是门岗ID 自然类型就为 3：进厂门岗
//            hashMap.put("queueLocationId", map.get("gatepostId"));
//            hashMap.put("gridType", 3);
//        }else if(map.get("platformId") != null){ // 1：成品仓库
//            hashMap.put("queueLocationId", map.get("platformId"));
//            hashMap.put("gridType", 1);
//        }
//        //通过门岗ID和 类型 来查询网格ID
//        BigDecimal gridId = queuingRulesService.getGridIdByGatepostIdAndGridType(hashMap);
//        map.put("gridId", gridId);
//        List<Map<String, Object>> listQueueMes = qmsQueueListMapper.getListQueueMes(map);
//        for (Map<String, Object> queueMes : listQueueMes) {
//            //获取排队开始时间
//            Date resultStartTime = (Date) queueMes.get("resultStartTime");
//            long sTime = new Date().getTime() - resultStartTime.getTime(); //当前时间与开始时间相差毫秒数
//            sTime = sTime / (1000 * 60);
//            queueMes.put("minute", sTime + "分钟");
//        }
//        return listQueueMes;
//    }
}
